#include <Fonts.h>
#include <MT12864.h>

//******************************************************************************
//***       *******************************************************
//******************************************************************************
void buff_clear(void)
{
    unsigned char i, j;
    for (i=0; i<8; i++)
    {
        for (j=0; j<128; j++) LCD_Buf[i][j]=0;
    }
}

//******************************************************************************
//***    .         ***********
//***     -  ...    .   ***********
//******************************************************************************
void lcd_pixel(unsigned char x, unsigned char y, unsigned char color)
{
    unsigned char i, j;
    i = y/8;
    j = y%8;
    if(color) LCD_Buf[i][x] = LCD_Buf[i][x] | (1<<(j));
    else LCD_Buf[i][x] = LCD_Buf[i][x] & (~(1<<(j)));
}

//******************************************************************************
//***       **********************************************************
//******************************************************************************
void lcd_line( char x1, char y1, char x2, char y2, char mode )
{
    int dx, dy, stepx, stepy, fraction;
    dy = y2 - y1;
    dx = x2 - x1;
    if(dy < 0)
    {
        dy    = -dy;
        stepy = -1;
    }
    else
    {
        stepy = 1;
    }
    if( dx < 0 )
    {
        dx    = -dx;
        stepx = -1;
    }
    else
    {
        stepx = 1;
    }
    dx <<= 1;
    dy <<= 1;
    lcd_pixel(x1, y1, mode);
    if(dx > dy)
    {
        fraction = dy - (dx >> 1);
        while(x1 != x2)
        {
            if (fraction >= 0)
            {
                y1 += stepy;
                fraction -= dx;
            }
            x1 += stepx;
            fraction += dy;
            lcd_pixel(x1, y1, mode);
        }
    }
    else
    {
        fraction = dx - (dy >> 1);
        while(y1 != y2)
        {
            if(fraction >= 0)
            {
                x1 += stepx;
                fraction -= dy;
            }
            y1 += stepy;
            fraction += dx;
            lcd_pixel(x1, y1, mode);
        }
    }
}

//******************************************************************************
//***         *************************************************
//******************************************************************************
void lcd_dashline(char x1, char y1, char x2, char y2)
{
    char mode;
    int dx, dy, stepx, stepy, fraction;
    dy = y2 - y1;
    dx = x2 - x1;
    mode = 1;

    if(dy < 0)
    {
        dy    = -dy;
        stepy = -1;
    }
    else
    {
        stepy = 1;
    }
    if(dx < 0)
    {
        dx    = -dx;
        stepx = -1;
    }
    else
    {
        stepx = 1;
    }
    dx <<= 1;
    dy <<= 1;
    lcd_pixel(x1, y1, mode);
    if(dx > dy)
    {
        fraction = dy - (dx >> 1);
        while(x1 != x2)
        {
            if(fraction >= 0)
            {
                y1 += stepy;
                fraction -= dx;
            }
            x1 += stepx;
            fraction += dy;

            if(mode) mode=0;
            else mode=1;

            lcd_pixel(x1, y1, mode);
        }
    }
    else
    {
        fraction = dx - (dy >> 1);
        while(y1 != y2)
        {
            if(fraction >= 0)
            {
                x1 += stepx;
                fraction -= dy;
            }
            y1 += stepy;
            fraction += dx;

            if(mode) mode = 0;
            else mode = 1;

            lcd_pixel(x1, y1, mode);
        }
    }
}

//******************************************************************************
//***        ************************************************
//******************************************************************************
void lcd_rectangle(char x, char y, char x1, char y1, unsigned char color)
{
    lcd_line (x,y,x,y1,color);
    lcd_line (x,y1,x1,y1,color);
    lcd_line (x1,y1,x1,y,color);
    lcd_line (x,y,x1,y,color);
}

//******************************************************************************
//***        ************************************
//******************************************************************************
void lcd_fillrect(char x, char y, char x1, char y1, unsigned char color)
{
    unsigned char i;
    for(i=y; i <= y1; i++) lcd_line(x, i, x1, i, color);
}

//******************************************************************************
//***        ******************************************
//******************************************************************************
lcd_setxy(char x, unsigned char y)
{
    LCD_Pos_X = x;
    LCD_Pos_Y = y;
}

//******************************************************************************
//***       ********************************************************
//******************************************************************************
void lcd_deg12x16(unsigned char c, unsigned char color)
{
     char i, j;
    unsigned char b;

    // c = address of character in the font array
    for(i=0; i < 12; i++)
    {
        b = deg12x16[c][i];
        for(j=0; j < 8; j++)
        {
            lcd_pixel(LCD_Pos_X, LCD_Pos_Y+j, (~(b ^ color)) & 1);
            b >>= 1;
        }
        LCD_Pos_X++;
    }

    LCD_Pos_Y += 8;
    LCD_Pos_X -= 12;
    for(i=12; i<24; i++)
    {
        b = deg12x16[c][i];
        for(j=0; j < 8; j++)
        {
            lcd_pixel(LCD_Pos_X, LCD_Pos_Y+j, (~(b ^ color)) & 1);
            b >>= 1;
        }
        LCD_Pos_X++;
    }
}

//******************************************************************************
//***       ********************************************************
//******************************************************************************
void lcd_char12x16(unsigned char c, unsigned char color)
{
    char i, j;
    unsigned char b;

    // c = address of character in the font array
    for(i=0; i < 12; i++)
    {
        b = Font_12x16[c][i];
        for(j=0; j < 8; j++)
        {
            lcd_pixel(LCD_Pos_X, LCD_Pos_Y+j, (~(b ^ color)) & 1);
            b >>= 1;
        }
        LCD_Pos_X++;
    }
    LCD_Pos_Y += 8;
    LCD_Pos_X -= 12;
    for(i=12; i < 24; i++)
    {
        b = Font_12x16[c][i];
        for(j=0; j < 8; j++)
        {
            lcd_pixel(LCD_Pos_X, LCD_Pos_Y+j, (~(b ^ color)) & 1);
            b >>= 1;
        }
        LCD_Pos_X++;
    }
    LCD_Pos_Y -= 8;
}

//******************************************************************************
//***       ********************************************************
//******************************************************************************
void lcd_char(unsigned char c, unsigned char color)
{
    char i, j;
    unsigned char b;

    c = c-32;
    // c = address of character in the font array
    for(i=0; i < 8; i++)
    {
        b = font8x8[c][i];
        for(j=0; j < 8; j++)
        {
            lcd_pixel(LCD_Pos_X, LCD_Pos_Y+j, (~(b ^ color)) & 1);
            b >>= 1;
        }
        LCD_Pos_X++;
    }
}

//******************************************************************************
//***          ***********************************
//******************************************************************************
void lcd_wrs(unsigned char * str, unsigned char color)
{
    while(*str !='\0') lcd_char(*str++, color);
}

//******************************************************************************
//***          ******************************************
//******************************************************************************
void lcd_wrsf(unsigned char flash * str, unsigned char color)
{
   while(*str !='\0') lcd_char(*str++, color);
}

//******************************************************************************
//***          ***********************************
//******************************************************************************
void lcd_wrs16(unsigned char * str, unsigned char color)
{
   while(*str !='\0')  lcd_char12x16(*str++, color);
}

//******************************************************************************
//***          ******************************************
//******************************************************************************
void lcd_wrsf16(unsigned char flash * str, unsigned char color)
{
   while(*str !='\0')  lcd_char12x16(*str++, color);
}

//******************************************************************************
//***          ****************************************
//******************************************************************************
void lcd_pict(unsigned char flash * p_arr, unsigned char W, unsigned char H, unsigned char X, unsigned char Y, unsigned char color)
{
    unsigned int i, j, k;

    for(i=0; i < H; i++)
        for(j=0; j < W; j++)
            for(k=0; k < 8; k++) lcd_pixel(X+j, Y+i*8+k, ~((p_arr[j+i*W]>>k)^color)&1);
}
