#include <mega128.h>
#include <delay.h>
#include "Fonts.h"

//******************************************************************************
//***      E  ().                           ***
//***    1  ,  PROTEUS .   LCD_E             ***
//***    inline ,    1  .               ***
//***       .                           ***
//******************************************************************************
#define Del 2

//******************************************************************************
//***    E  ().                                                 ***
//***     - . PORTG.1  :(                   ***
//******************************************************************************
#define LCD_E(x) PORTG = (PORTG & (~(1 << 1))) | (x << 1)
/*
inline void LCD_E(unsigned char x)
{
    PORTG = (PORTG & (~(1 << 1))) | (x << 1);
}
*/

//******************************************************************************
//***    A0  (/).                                       ***
//***   "1"  , "0"  . PORTG.0  :(    ***
//******************************************************************************
#define LCD_A0(x) PORTG = (PORTG & (~(1 << 0))) | (x << 0)
/*
inline void LCD_A0(unsigned char x)
{
    PORTG = (PORTG & (~(1 << 0))) | (x << 0);
}
*/

//******************************************************************************
//***    RW  (/).                                        ***
//***  "1" -  , "0" -  .                                 ***
//******************************************************************************
#define LCD_RW PORTD.7

//******************************************************************************
//***    CS2  ( 2).                                           ***
//***    - .                                             ***
//******************************************************************************
#define LCD_CS2 PORTB.7

//******************************************************************************
//***    CS  ().                                              ***
//***     - .                                           ***
//******************************************************************************
#define LCD_CS PORTB.6

//******************************************************************************
//***    RES  ().                                               ***
//***     - 1- .  - .                         ***
//******************************************************************************
#define LCD_RES PORTD.6

//******************************************************************************
//***    DATA  ().   **********************************************
//******************************************************************************
#define LCD_DATA              PORTC

//******************************************************************************
//***           ***************************************
//******************************************************************************
unsigned char LCD_Buf[8][128];

//******************************************************************************
//***        *******************************************************
//******************************************************************************
unsigned char LCD_Pos_X, LCD_Pos_Y;

//******************************************************************************
//***         ********************************************
//******************************************************************************
void LCD_Cmd(unsigned char Chip, unsigned char Cmd)
{
    LCD_CS = Chip;
    LCD_A0(0);
    LCD_DATA = Cmd;
    delay_us(Del);
    LCD_E(1);
    delay_us(Del);
    LCD_E(0);
}

//******************************************************************************
//***         *********************************************
//******************************************************************************
void LCD_Dat(char Chip, char Data)
{
    LCD_CS = Chip;
    LCD_A0(1);
    LCD_DATA = Data;
    delay_us(Del);
    LCD_E(1);
    delay_us(Del);
    LCD_E(0);
}

//******************************************************************************
//***   /     ***********************************
//******************************************************************************
inline void LCD_On_Off(char Chip, char Data)
{
    LCD_Cmd(Chip, 0b00111110 + Data);
}

//******************************************************************************
//***          ***********************************
//******************************************************************************
inline void LCD_Set_First_Str(char Chip, char Data)
{
    LCD_Cmd(Chip, 0b11000000 + Data);
}

//******************************************************************************
//***          *************************************
//******************************************************************************
inline void LCD_Set_Page(char Chip, char Data)
{
    LCD_Cmd(Chip, 0b10111000 + Data);
}

//******************************************************************************
//***           ***********************
//******************************************************************************
inline void LCD_Set_Col(char Chip, char Data)
{
    LCD_Cmd(Chip, 0b01000000 + Data);
}

//******************************************************************************
//***        ******************************************
//******************************************************************************
void LCD_Init(void)
{
    LCD_RW = 0;
    LCD_CS = 0;
    delay_us(100);
    LCD_RES = 1;
    delay_us(1000);
    LCD_Set_First_Str(0, 0);
    LCD_Set_First_Str(1, 0);
    LCD_Set_Page(0, 0);
    LCD_Set_Page(1, 0);
    LCD_Set_Col(0, 0);
    LCD_Set_Col(1, 0);
    LCD_On_Off(0, 1);
    LCD_On_Off(1, 1);
}

//******************************************************************************
//***        ************************************************
//******************************************************************************
void LCD_Clear(void)
{
    unsigned char i, j;
    for(j=0; j < 8; j++)
    {
        LCD_Set_Page(0, j);
        LCD_Set_Col(0, 0);
        LCD_Set_Page(1, j);
        LCD_Set_Col(1, 0);

        for(i=0; i < 64; i++) LCD_Dat(0, 0);
        for(i=0; i < 64; i++) LCD_Dat(1, 0);
    }
}

//******************************************************************************
//***    .        ********************
//******************************************************************************
void LCD_Refresh(void)
{
    unsigned char i, j;

    LCD_Set_First_Str(0, 0);
    LCD_Set_First_Str(1, 0);
    LCD_Set_Page(0, 0);
    LCD_Set_Page(1, 0);
    LCD_Set_Col(0, 0);
    LCD_Set_Col(1, 0);

    for(i=0; i < 8; i++)
    {
        LCD_Set_Page(0, i);
        LCD_Set_Col(0, 0);
        LCD_A0(1);
        LCD_CS = 0;
        for(j=64; j < 128; j++)
        {
            LCD_DATA = LCD_Buf[i,j];
            delay_us(Del);
            LCD_E(1);
            delay_us(Del);
            LCD_E(0);
        }
    }

    for(i=0; i < 8; i++)
    {
        LCD_Set_Page(1, i);
        LCD_Set_Col(1, 0);
        LCD_A0(1);
        LCD_CS = 1;

        for(j=0; j < 64; j++)
        {
            LCD_DATA = LCD_Buf[i,j];
            delay_us(Del);
            LCD_E(1);
            delay_us(Del);
            LCD_E(0);
        }
    }
}